using System;
using System.Collections.Generic;
using System.IO;
using System.Text;
using Microsoft.Xna.Framework.Storage;

namespace LD10Tanks
{
    public class Highscores
    {
        /// <summary>
        /// Unique level id -> score map.
        /// </summary>
        private Dictionary<int, float> m_id_score_map = new Dictionary<int,float>();

        private string SaveFilename
        {
            get { return Path.Combine(StorageContainer.TitleLocation, "save.dat"); }
        }

        public Highscores()
        {
            // Try to load the old scores
            try
            {
                using (BinaryReader fin = new BinaryReader(File.Open(SaveFilename, FileMode.Open, FileAccess.Read)))
                {
                    int pairs = fin.ReadInt32();
                    for (int i = 0; i < pairs; ++i)
                    {
                        int uid = fin.ReadInt32();
                        float score = fin.ReadSingle();

                        m_id_score_map[uid] = score;
                    }

                    fin.Close();
                }
            }
            catch
            {
            }
        }

        public void Save()
        {
            // Try to save the new scores
            try
            {
                using (BinaryWriter fout = new BinaryWriter(File.Open(SaveFilename, FileMode.Create, FileAccess.Write)))
                {
                    fout.Write((int)m_id_score_map.Count);
                    foreach (int uid in m_id_score_map.Keys)
                    {
                        fout.Write((int)uid);
                        fout.Write((float)m_id_score_map[uid]);
                    }

                    fout.Close();
                }
            }
            catch
            {
            }
        }

        /// <summary>
        /// Returns the levels highscore time or -1.
        /// </summary>
        public float GetLevelScore(int level_id)
        {
            if (!m_id_score_map.ContainsKey(level_id))
                return -1;
            return m_id_score_map[level_id];
        }

        /// <summary>
        /// Records a new level score. Returns true if it was a new highscore.
        /// </summary>
        public bool RecordScore(int level_id, float score)
        {
            float old_score = GetLevelScore(level_id);

            if (old_score >= 0.0f)
            {
                if (score < old_score)
                {
                    m_id_score_map[level_id] = score;
                }
                return false;
            }

            // First score
            m_id_score_map[level_id] = score;
            return true;
        }
    }
}
