using System;
using System.Collections.Generic;
using System.Text;
using System.Xml;
using System.Windows.Forms;

namespace TankLevelEditor
{
    public class LevelList
    {
        public List<Level> levels = new List<Level>();

        public LevelList()
        {
            try
            {
                XmlDocument xml = new XmlDocument();
                xml.Load("levels.xml");

                foreach (XmlElement level_node in xml["levels"].SelectNodes("level"))
                {
                    Level level = new Level();
                    levels.Add(level);

                    level.Name = GetStringValue(level_node["name"], level.Name);
                    level.UniqueId = GetIntValue(level_node["unique_id"], level.UniqueId);

                    foreach (XmlElement node in level_node.SelectNodes("tank"))
                    {
                        Tank tank = new Tank();
                        level.tanks.Add(tank);

                        tank.pos.X = GetFloatValue(node["x"], tank.pos.X);
                        tank.pos.Y = GetFloatValue(node["y"], tank.pos.Y);
                        tank.FixedPosition = GetIntValue(node["fixed_position"], tank.FixedPosition ? 1 : 0) != 0;
                        tank.Bullets = GetIntValue(node["bullets"], tank.Bullets);
                        tank.Type = (TankType)GetIntValue(node["type"], (int)tank.Type);
                    }

                    foreach (XmlElement node in level_node.SelectNodes("wall"))
                    {
                        Wall wall = new Wall();
                        level.walls.Add(wall);

                        wall.pt1.X = GetFloatValue(node["x1"], wall.pt1.X);
                        wall.pt1.Y = GetFloatValue(node["y1"], wall.pt1.Y);
                        wall.pt2.X = GetFloatValue(node["x2"], wall.pt2.X);
                        wall.pt2.Y = GetFloatValue(node["y2"], wall.pt2.Y);
                        wall.Thickness = GetFloatValue(node["thickness"], wall.Thickness);
                        wall.Type = (WallType)GetIntValue(node["type"], (int)wall.Type);
                    }
                }
            }
            catch
            {
            }
        }

        public void Save()
        {
            try
            {
                XmlDocument xml = new XmlDocument();
                XmlElement levels_node = xml.CreateElement("levels");
                xml.AppendChild(levels_node);

                foreach (Level level in levels)
                {
                    XmlElement level_node = xml.CreateElement("level");
                    levels_node.AppendChild(level_node);

                    PutValue(level_node, "name", level.Name);
                    PutValue(level_node, "unique_id", level.UniqueId);

                    foreach (Tank tank in level.tanks)
                    {
                        XmlElement tank_node = xml.CreateElement("tank");
                        level_node.AppendChild(tank_node);

                        PutValue(tank_node, "x", tank.pos.X);
                        PutValue(tank_node, "y", tank.pos.Y);
                        PutValue(tank_node, "fixed_position", tank.FixedPosition ? 1 : 0);
                        PutValue(tank_node, "bullets", tank.Bullets);
                        PutValue(tank_node, "type", (int)tank.Type);
                    }

                    foreach (Wall wall in level.walls)
                    {
                        XmlElement wall_node = xml.CreateElement("wall");
                        level_node.AppendChild(wall_node);

                        PutValue(wall_node, "x1", wall.pt1.X);
                        PutValue(wall_node, "y1", wall.pt1.Y);
                        PutValue(wall_node, "x2", wall.pt2.X);
                        PutValue(wall_node, "y2", wall.pt2.Y);
                        PutValue(wall_node, "thickness", wall.Thickness);
                        PutValue(wall_node, "type", (int)wall.Type);
                    }
                }

                xml.Save("levels.xml");
            }
            catch (Exception e)
            {
                Console.WriteLine(e);
                MessageBox.Show(e.ToString(), "Error saving!", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private string GetStringValue(XmlElement node, string def)
        {
            if (node == null)
                return def;
            string ret = node.GetAttribute("value");
            if (ret == null)
                return def;
            return ret;
        }

        private int GetIntValue(XmlElement node, int def)
        {
            try
            {
                return Int32.Parse(GetStringValue(node, null));
            }
            catch
            {
                return def;
            }
        }

        private float GetFloatValue(XmlElement node, float def)
        {
            int def_int = (int)(def * 65536.0f);
            int ret = GetIntValue(node, def_int);
            return (float)ret / 65536.0f;
        }

        private void PutValue(XmlElement owner, string name, string val)
        {
            XmlElement node = owner.OwnerDocument.CreateElement(name);
            owner.AppendChild(node);
            node.SetAttribute("value", val);
        }

        private void PutValue(XmlElement owner, string name, int val)
        {
            PutValue(owner, name, val.ToString());
        }

        private void PutValue(XmlElement owner, string name, float val)
        {
            PutValue(owner, name, (int)(val * 65536.0f));
        }
    }
}
