using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace LD10Tanks
{
    public class TankMeshes
    {
        private const float MainRadius = 0.5f;
        private const float BaseHeight = 0.1f;
        private const float TopHeight = 0.1f;

        private const float BulletRadius = 0.1f;

        private const float GunAngle = 45.0f;

        private const float GenCellSize = 0.03f;//0.008f;

        public const int MaxGuns = 5;

        public VertexPositionNormalColor[] meshvtx_base;
        public int[] meshidx_base;

        public VertexPositionNormalColor[][] meshvtx_top;
        public int[][] meshidx_top;

        public static float gun_height;
        public static Vector3[][] gun_dirs;

        public VertexPositionNormalColor[] meshvtx_bullet;
        public int[] meshidx_bullet;

        public TankMeshes()
        {
            const float x_size = MainRadius * 2.0f;
            const float y_size = BaseHeight + TopHeight * 4;
            const float z_size = MainRadius * 2.0f;

            MarchingCubes tri_gen = new MarchingCubes(new BoundingBox(new Vector3(-x_size, -GenCellSize * 3, -z_size), new Vector3(x_size, y_size, z_size)), GenCellSize);
            EvalLevel level = new EvalLevel();

            AddBase(level);
            Generate(tri_gen, level, out meshvtx_base, out meshidx_base);

            meshvtx_top = new VertexPositionNormalColor[MaxGuns][];
            meshidx_top = new int[MaxGuns][];
            gun_dirs = new Vector3[MaxGuns][];
            for (int i = 0; i < MaxGuns; ++i)
            {
                level.Clear();
                AddTop(level, i + 1, out gun_dirs[i]);
                Generate(tri_gen, level, out meshvtx_top[i], out meshidx_top[i]);
            }

            level.Clear();
            AddBullet(level);
            Generate(tri_gen, level, out meshvtx_bullet, out meshidx_bullet);
            for (int i = 0; i < meshvtx_bullet.Length; ++i)
            {
                meshvtx_bullet[i].Color = new Color(252, 225, 17);
            }

            tri_gen = null;
            GC.Collect();
        }

        private void AddBase(EvalLevel level)
        {
            level.FieldSolid.AddCylinder(new Vector3(0, -0.1f, 0), new Vector3(0, BaseHeight, 0), MainRadius, MainRadius * 0.1f);
            level.StartPoints.Add(new Vector3(0, 0, 0));
        }

        private void AddTop(EvalLevel level, int guns, out Vector3[] gun_dir)
        {
            const float top_radius = MainRadius * 0.2f;

            float top_plate_top = BaseHeight + TopHeight * 0.2f;

            // The main base plate
            level.FieldSolid.AddCylinder(new Vector3(0, BaseHeight, 0), new Vector3(0, top_plate_top, 0), top_radius, top_radius);
            level.StartPoints.Add(new Vector3(0, (BaseHeight + top_plate_top) * 0.5f, 0));

            // Add the guns
            const float gun_radius = TopHeight * 0.5f;
            gun_height = top_plate_top + gun_radius;

            float gun_ang = -MathHelper.ToRadians(90.0f + GunAngle * (guns - 1) * 0.5f);
            float gun_ang_add = MathHelper.ToRadians(GunAngle);

            gun_dir = new Vector3[guns];

            for (int i = 0; i < guns; ++i, gun_ang += gun_ang_add)
            {
                const float rad1 = 0.0f;
                const float rad2 = MainRadius * 0.9f;
                float x1 = (float)Math.Cos(gun_ang) * rad1;
                float z1 = (float)Math.Sin(gun_ang) * rad1;
                float x2 = (float)Math.Cos(gun_ang) * rad2;
                float z2 = (float)Math.Sin(gun_ang) * rad2;

                gun_dir[i] = new Vector3((float)Math.Cos(gun_ang), 0, (float)Math.Sin(gun_ang));

                float barrel_y = gun_height;
                level.FieldSolid.AddCylinder(new Vector3(x1, barrel_y, z1), new Vector3(x2, barrel_y, z2), gun_radius, gun_radius, false);
            }

            // Add a top cap
            {
                const float cap_rad1 = top_radius * 0.7f;
                const float cap_rad2 = top_radius * 0.2f;
                float cap_top_y = top_plate_top + TopHeight * 1.9f;
                float cap_top_x = -top_radius * 1.7f;

                level.FieldSolid.AddCylinder(new Vector3(0, top_plate_top, 0), new Vector3(cap_top_x, cap_top_y, 0), cap_rad1, cap_rad2, true);
            }
        }

        private void AddBullet(EvalLevel level)
        {
            level.StartPoints.Add(Vector3.Zero);
            level.FieldSolid.AddSphere(Vector3.Zero, BulletRadius);
        }

        private void AddBaseHighPoly(EvalLevel level)
        {
            float from_ground = BaseHeight * 0.4f;
            float base_plate_top = from_ground + BaseHeight * 0.2f;

            // The main base plate
            level.FieldSolid.AddCylinder(new Vector3(0, from_ground, 0), new Vector3(0, base_plate_top, 0), MainRadius, MainRadius);
            level.StartPoints.Add(new Vector3(0, (from_ground + base_plate_top) * 0.5f, 0));

            // Ground supports
            const int support_count = 5;
            for (int i = 0; i < support_count; ++i)
            {
                float ang = ((float)i / (float)support_count) * (float)Math.PI * 2.0f;
                float x = (float)Math.Cos(ang) * MainRadius;
                float z = (float)Math.Sin(ang) * MainRadius;

                // The main pilar
                float top = base_plate_top + BaseHeight * 0.2f;
                level.FieldSolid.AddCylinder(new Vector3(x, 0, z), new Vector3(x, top, z), 0.1f, 0.05f, true);

                // An added ornament cylinder on top
                float top2 = top + BaseHeight * 0.1f;
                level.FieldSolid.AddCylinder(new Vector3(x, top, z), new Vector3(x * 1.01f, top2, z * 1.01f), 0.07f, 0.07f);
            }

            // Tanks on top
            const int tank_count = 17;
            for (int i = 0; i < tank_count; ++i)
            {
                float ang = ((float)i / (float)tank_count) * (float)Math.PI * 2.0f;
                const float rad1 = MainRadius * 0.5f;
                const float rad2 = MainRadius * 0.8f;
                float x1 = (float)Math.Cos(ang) * rad1;
                float z1 = (float)Math.Sin(ang) * rad1;
                float x2 = (float)Math.Cos(ang) * rad2;
                float z2 = (float)Math.Sin(ang) * rad2;

                const float tank_rad = BaseHeight * 0.1f;
                level.FieldSolid.AddCylinder(new Vector3(x1, base_plate_top + tank_rad, z1), new Vector3(x2, base_plate_top + tank_rad * 0.1f, z2), tank_rad, tank_rad, true);
            }

            // The main support
            {
                float support_y = base_plate_top * 0.0f;
                level.FieldSolid.AddSphere(new Vector3(0, support_y, 0), BaseHeight - support_y);
            }
        }

        private void AddTopHighPoly(EvalLevel level, int guns)
        {
            const float top_radius = MainRadius * 0.2f;

            float top_plate_top = BaseHeight + TopHeight * 0.2f;

            // The main base plate
            level.FieldSolid.AddCylinder(new Vector3(0, BaseHeight, 0), new Vector3(0, top_plate_top, 0), top_radius, top_radius);
            level.StartPoints.Add(new Vector3(0, (BaseHeight + top_plate_top) * 0.5f, 0));

            // Add the guns
            const float gun_radius = TopHeight * 0.3f;
            const float gun_radius_inner = gun_radius * 0.6f;

            float gun_ang = -MathHelper.ToRadians(GunAngle * (guns - 1) * 0.5f);
            float gun_ang_add = MathHelper.ToRadians(GunAngle);

            for (int i = 0; i < guns; ++i, gun_ang += gun_ang_add)
            {
                const float rad1 = 0.0f;
                const float rad2 = MainRadius * 0.7f;
                float x1 = (float)Math.Cos(gun_ang) * rad1;
                float z1 = (float)Math.Sin(gun_ang) * rad1;
                float x2 = (float)Math.Cos(gun_ang) * rad2;
                float z2 = (float)Math.Sin(gun_ang) * rad2;

                float barrel_y = top_plate_top + gun_radius;
                level.FieldSolid.AddCylinder(new Vector3(x1, barrel_y, z1), new Vector3(x2, barrel_y, z2), gun_radius, gun_radius, false);

                // Add the barrel hole
                level.FieldExclude.AddCylinder(new Vector3(x1, barrel_y, z1), new Vector3(x2, barrel_y, z2), gun_radius_inner, gun_radius_inner, true);
            }

            // Add a top cap
            {
                const float cap_rad1 = top_radius * 0.7f;
                const float cap_rad2 = top_radius * 0.2f;
                float cap_top_y = top_plate_top + TopHeight * 0.9f;
                float cap_top_x = -top_radius * 1.2f;

                level.FieldSolid.AddCylinder(new Vector3(0, top_plate_top, 0), new Vector3(cap_top_x, cap_top_y, 0), cap_rad1, cap_rad2, true);
            }
        }

        private void Generate(MarchingCubes generator, EvalLevel level, out VertexPositionNormalColor[] vtx, out int[] idx)
        {
            generator.Generate(level);
            vtx = generator.Vertices.ToArray();
            idx = generator.Triangles.ToArray();
        }
    }
}
